aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
-rw-r--r--Makefile8
-rw-r--r--cache.c6
-rw-r--r--cgit.c41
-rw-r--r--cgit.css2
-rw-r--r--cgit.h10
-rw-r--r--cgitrc.5.txt13
-rw-r--r--cmd.c20
-rwxr-xr-xfilters/commit-links.sh2
-rw-r--r--filters/email-gravatar.lua17
-rw-r--r--filters/email-libravatar.lua17
-rw-r--r--filters/file-authentication.lua31
-rw-r--r--filters/gentoo-ldap-authentication.lua31
-rwxr-xr-xfilters/html-converters/md2html24
-rw-r--r--filters/simple-authentication.lua31
m---------git0
-rw-r--r--parsing.c12
-rw-r--r--shared.c5
-rwxr-xr-xtests/setup.sh14
-rwxr-xr-xtests/t0001-validate-git-versions.sh10
-rwxr-xr-xtests/t0105-commit.sh2
-rwxr-xr-xtests/t0107-snapshot.sh125
-rwxr-xr-xtests/t0109-gitconfig.sh8
-rw-r--r--ui-atom.c4
-rw-r--r--ui-atom.h2
-rw-r--r--ui-blame.c27
-rw-r--r--ui-blob.c16
-rw-r--r--ui-clone.c4
-rw-r--r--ui-commit.c16
-rw-r--r--ui-diff.c18
-rw-r--r--ui-log.c57
-rw-r--r--ui-log.h2
-rw-r--r--ui-patch.c13
-rw-r--r--ui-plain.c15
-rw-r--r--ui-refs.c2
-rw-r--r--ui-repolist.c4
-rw-r--r--ui-shared.c120
-rw-r--r--ui-shared.h1
-rw-r--r--ui-snapshot.c41
-rw-r--r--ui-ssdiff.c23
-rw-r--r--ui-stats.c50
-rw-r--r--ui-summary.c2
-rw-r--r--ui-summary.h2
-rw-r--r--ui-tag.c14
-rw-r--r--ui-tree.c31
44 files changed, 574 insertions, 319 deletions
diff --git a/Makefile b/Makefile
index 05ea71f..d13c5bd 100644
--- a/Makefile
+++ b/Makefile
@@ -1,6 +1,6 @@
all::
-CGIT_VERSION = v1.2.1
+CGIT_VERSION = v1.2.3
CGIT_SCRIPT_NAME = cgit.cgi
CGIT_SCRIPT_PATH = /var/www/htdocs/cgit
CGIT_DATA_PATH = $(CGIT_SCRIPT_PATH)
@@ -14,8 +14,8 @@ htmldir = $(docdir)
pdfdir = $(docdir)
mandir = $(prefix)/share/man
SHA1_HEADER = <openssl/sha.h>
-GIT_VER = 2.18.0
-GIT_URL = https://www.kernel.org/pub/software/scm/git/git-$(GIT_VER).tar.gz
+GIT_VER = 2.32.0
+GIT_URL = https://www.kernel.org/pub/software/scm/git/git-$(GIT_VER).tar.xz
INSTALL = install
COPYTREE = cp -r
MAN5_TXT = $(wildcard *.5.txt)
@@ -157,7 +157,7 @@ clean-doc:
$(RM) cgitrc.5 cgitrc.5.html cgitrc.5.pdf cgitrc.5.xml cgitrc.5.fo
get-git:
- curl -L $(GIT_URL) | tar -xzf - && rm -rf git && mv git-$(GIT_VER) git
+ curl -L $(GIT_URL) | tar -xJf - && rm -rf git && mv git-$(GIT_VER) git
tags:
$(QUIET_TAGS)find . -name '*.[ch]' | xargs ctags
diff --git a/cache.c b/cache.c
index 2c70be7..55199e8 100644
--- a/cache.c
+++ b/cache.c
@@ -401,12 +401,12 @@ int cache_process(int size, const char *path, const char *key, int ttl,
static char *sprintftime(const char *format, time_t time)
{
static char buf[64];
- struct tm *tm;
+ struct tm tm;
if (!time)
return NULL;
- tm = gmtime(&time);
- strftime(buf, sizeof(buf)-1, format, tm);
+ gmtime_r(&time, &tm);
+ strftime(buf, sizeof(buf)-1, format, &tm);
return buf;
}
diff --git a/cgit.c b/cgit.c
index 6301b87..08d81a1 100644
--- a/cgit.c
+++ b/cgit.c
@@ -19,6 +19,16 @@
const char *cgit_version = CGIT_VERSION;
+__attribute__((constructor))
+static void constructor_environment()
+{
+ /* Do not look in /etc/ for gitconfig and gitattributes. */
+ setenv("GIT_CONFIG_NOSYSTEM", "1", 1);
+ setenv("GIT_ATTR_NOSYSTEM", "1", 1);
+ unsetenv("HOME");
+ unsetenv("XDG_CONFIG_HOME");
+}
+
static void add_mimetype(const char *name, const char *value)
{
struct string_list_item *item;
@@ -50,6 +60,8 @@ static void repo_config(struct cgit_repo *repo, const char *name, const char *va
repo->extra_head_content = xstrdup(value);
else if (!strcmp(name, "snapshots"))
repo->snapshots = ctx.cfg.snapshots & cgit_parse_snapshots_mask(value);
+ else if (!strcmp(name, "enable-blame"))
+ repo->enable_blame = atoi(value);
else if (!strcmp(name, "enable-commit-graph"))
repo->enable_commit_graph = atoi(value);
else if (!strcmp(name, "enable-log-filecount"))
@@ -312,11 +324,11 @@ static void querystring_cb(const char *name, const char *value)
ctx.qry.head = xstrdup(value);
ctx.qry.has_symref = 1;
} else if (!strcmp(name, "id")) {
- ctx.qry.sha1 = xstrdup(value);
- ctx.qry.has_sha1 = 1;
+ ctx.qry.oid = xstrdup(value);
+ ctx.qry.has_oid = 1;
} else if (!strcmp(name, "id2")) {
- ctx.qry.sha2 = xstrdup(value);
- ctx.qry.has_sha1 = 1;
+ ctx.qry.oid2 = xstrdup(value);
+ ctx.qry.has_oid = 1;
} else if (!strcmp(name, "ofs")) {
ctx.qry.ofs = atoi(value);
} else if (!strcmp(name, "path")) {
@@ -563,18 +575,13 @@ static void prepare_repo_env(int *nongit)
/* The path to the git repository. */
setenv("GIT_DIR", ctx.repo->path, 1);
- /* Do not look in /etc/ for gitconfig and gitattributes. */
- setenv("GIT_CONFIG_NOSYSTEM", "1", 1);
- setenv("GIT_ATTR_NOSYSTEM", "1", 1);
- unsetenv("HOME");
- unsetenv("XDG_CONFIG_HOME");
-
/* Setup the git directory and initialize the notes system. Both of these
* load local configuration from the git repository, so we do them both while
* the HOME variables are unset. */
setup_git_directory_gently(nongit);
- init_display_notes(NULL);
+ load_display_notes(NULL);
}
+
static int prepare_repo_cmd(int nongit)
{
struct object_id oid;
@@ -645,7 +652,7 @@ static inline void open_auth_filter(const char *function)
ctx.env.https ? ctx.env.https : "",
ctx.qry.repo ? ctx.qry.repo : "",
ctx.qry.page ? ctx.qry.page : "",
- ctx.qry.url ? ctx.qry.url : "",
+ cgit_currentfullurl(),
cgit_loginurl());
}
@@ -809,6 +816,8 @@ static void print_repo(FILE *f, struct cgit_repo *repo)
fprintf(f, "repo.homepage=%s\n", repo->homepage);
if (repo->clone_url)
fprintf(f, "repo.clone-url=%s\n", repo->clone_url);
+ fprintf(f, "repo.enable-blame=%d\n",
+ repo->enable_blame);
fprintf(f, "repo.enable-commit-graph=%d\n",
repo->enable_commit_graph);
fprintf(f, "repo.enable-log-filecount=%d\n",
@@ -983,9 +992,9 @@ static void cgit_parse_args(int argc, const char **argv)
} else if (skip_prefix(argv[i], "--head=", &arg)) {
ctx.qry.head = xstrdup(arg);
ctx.qry.has_symref = 1;
- } else if (skip_prefix(argv[i], "--sha1=", &arg)) {
- ctx.qry.sha1 = xstrdup(arg);
- ctx.qry.has_sha1 = 1;
+ } else if (skip_prefix(argv[i], "--oid=", &arg)) {
+ ctx.qry.oid = xstrdup(arg);
+ ctx.qry.has_oid = 1;
} else if (skip_prefix(argv[i], "--ofs=", &arg)) {
ctx.qry.ofs = atoi(arg);
} else if (skip_prefix(argv[i], "--scan-tree=", &arg) ||
@@ -1028,7 +1037,7 @@ static int calc_ttl(void)
if (!strcmp(ctx.qry.page, "snapshot"))
return ctx.cfg.cache_snapshot_ttl;
- if (ctx.qry.has_sha1)
+ if (ctx.qry.has_oid)
return ctx.cfg.cache_static_ttl;
if (ctx.qry.has_symref)
diff --git a/cgit.css b/cgit.css
index d4aadbf..dfa144d 100644
--- a/cgit.css
+++ b/cgit.css
@@ -561,7 +561,7 @@ div#cgit table.diff td div.del {
color: red;
}
-div#cgit .sha1 {
+div#cgit .oid {
font-family: monospace;
font-size: 90%;
}
diff --git a/cgit.h b/cgit.h
index 32dfd7a..69b5c13 100644
--- a/cgit.h
+++ b/cgit.h
@@ -8,12 +8,13 @@
#include <cache.h>
#include <grep.h>
#include <object.h>
+#include <object-store.h>
#include <tree.h>
#include <commit.h>
#include <tag.h>
#include <diff.h>
#include <diffcore.h>
-#include <argv-array.h>
+#include <strvec.h>
#include <refs.h>
#include <revision.h>
#include <log-tree.h>
@@ -93,6 +94,7 @@ struct cgit_repo {
char *logo_link;
char *snapshot_prefix;
int snapshots;
+ int enable_blame;
int enable_commit_graph;
int enable_log_filecount;
int enable_log_linecount;
@@ -162,7 +164,7 @@ struct reflist {
struct cgit_query {
int has_symref;
- int has_sha1;
+ int has_oid;
int has_difftype;
char *raw;
char *repo;
@@ -170,8 +172,8 @@ struct cgit_query {
char *search;
char *grep;
char *head;
- char *sha1;
- char *sha2;
+ char *oid;
+ char *oid2;
char *path;
char *name;
char *url;
diff --git a/cgitrc.5.txt b/cgitrc.5.txt
index 34b351b..33a6a8c 100644
--- a/cgitrc.5.txt
+++ b/cgitrc.5.txt
@@ -407,9 +407,12 @@ side-by-side-diffs::
snapshots::
Text which specifies the default set of snapshot formats that cgit
generates links for. The value is a space-separated list of zero or
- more of the values "tar", "tar.gz", "tar.bz2", "tar.xz" and "zip".
- The special value "all" enables all snapshot formats.
- Default value: none.
+ more of the values "tar", "tar.gz", "tar.bz2", "tar.lz", "tar.xz",
+ "tar.zst" and "zip". The special value "all" enables all snapshot
+ formats. Default value: none.
+ All compressors use default settings. Some settings can be influenced
+ with environment variables, for example set ZSTD_CLEVEL=10 in web
+ server environment for higher (but slower) zstd compression.
source-filter::
Specifies a command which will be invoked to format plaintext blobs
@@ -485,6 +488,10 @@ repo.email-filter::
Override the default email-filter. Default value: none. See also:
"enable-filter-overrides". See also: "FILTER API".
+repo.enable-blame::
+ A flag which can be used to disable the global setting
+ `enable-blame'. Default value: none.
+
repo.enable-commit-graph::
A flag which can be used to disable the global setting
`enable-commit-graph'. Default value: none.
diff --git a/cmd.c b/cmd.c
index 63f0ae5..0eb75b1 100644
--- a/cmd.c
+++ b/cmd.c
@@ -66,7 +66,7 @@ static void about_fn(void)
static void blame_fn(void)
{
- if (ctx.cfg.enable_blame)
+ if (ctx.repo->enable_blame)
cgit_print_blame();
else
cgit_print_error_page(403, "Forbidden", "Blame is disabled");
@@ -74,22 +74,22 @@ static void blame_fn(void)
static void blob_fn(void)
{
- cgit_print_blob(ctx.qry.sha1, ctx.qry.path, ctx.qry.head, 0);
+ cgit_print_blob(ctx.qry.oid, ctx.qry.path, ctx.qry.head, 0);
}
static void commit_fn(void)
{
- cgit_print_commit(ctx.qry.sha1, ctx.qry.path);
+ cgit_print_commit(ctx.qry.oid, ctx.qry.path);
}
static void diff_fn(void)
{
- cgit_print_diff(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path, 1, 0);
+ cgit_print_diff(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path, 1, 0);
}
static void rawdiff_fn(void)
{
- cgit_print_diff(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path, 1, 1);
+ cgit_print_diff(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path, 1, 1);
}
static void info_fn(void)
@@ -99,7 +99,7 @@ static void info_fn(void)
static void log_fn(void)
{
- cgit_print_log(ctx.qry.sha1, ctx.qry.ofs, ctx.cfg.max_commit_count,
+ cgit_print_log(ctx.qry.oid, ctx.qry.ofs, ctx.cfg.max_commit_count,
ctx.qry.grep, ctx.qry.search, ctx.qry.path, 1,
ctx.repo->enable_commit_graph,
ctx.repo->commit_sort);
@@ -125,7 +125,7 @@ static void repolist_fn(void)
static void patch_fn(void)
{
- cgit_print_patch(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path);
+ cgit_print_patch(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path);
}
static void plain_fn(void)
@@ -140,7 +140,7 @@ static void refs_fn(void)
static void snapshot_fn(void)
{
- cgit_print_snapshot(ctx.qry.head, ctx.qry.sha1, ctx.qry.path,
+ cgit_print_snapshot(ctx.qry.head, ctx.qry.oid, ctx.qry.path,
ctx.qry.nohead);
}
@@ -156,12 +156,12 @@ static void summary_fn(void)
static void tag_fn(void)
{
- cgit_print_tag(ctx.qry.sha1);
+ cgit_print_tag(ctx.qry.oid);
}
static void tree_fn(void)
{
- cgit_print_tree(ctx.qry.sha1, ctx.qry.path);
+ cgit_print_tree(ctx.qry.oid, ctx.qry.path);
}
#define def_cmd(name, want_repo, want_vpath, is_clone) \
diff --git a/filters/commit-links.sh b/filters/commit-links.sh
index 5881952..796ac30 100755
--- a/filters/commit-links.sh
+++ b/filters/commit-links.sh
@@ -19,7 +19,7 @@ regex=''
# This expression generates links to commits referenced by their SHA1.
regex=$regex'
-s|\b([0-9a-fA-F]{7,40})\b|<a href="./?id=\1">\1</a>|g'
+s|\b([0-9a-fA-F]{7,64})\b|<a href="./?id=\1">\1</a>|g'
# This expression generates links to a fictional bugtracker.
regex=$regex'
diff --git a/filters/email-gravatar.lua b/filters/email-gravatar.lua
index 52cf426..c39b490 100644
--- a/filters/email-gravatar.lua
+++ b/filters/email-gravatar.lua
@@ -3,15 +3,24 @@
-- prefix in filters. It is much faster than the corresponding python script.
--
-- Requirements:
--- luacrypto >= 0.3
--- <http://mkottman.github.io/luacrypto/>
+-- luaossl
+-- <http://25thandclement.com/~william/projects/luaossl.html>
--
-local crypto = require("crypto")
+local digest = require("openssl.digest")
+
+function md5_hex(input)
+ local b = digest.new("md5"):final(input)
+ local x = ""
+ for i = 1, #b do
+ x = x .. string.format("%.2x", string.byte(b, i))
+ end
+ return x
+end
function filter_open(email, page)
buffer = ""
- md5 = crypto.digest("md5", email:sub(2, -2):lower())
+ md5 = md5_hex(email:sub(2, -2):lower())
end
function filter_close()
diff --git a/filters/email-libravatar.lua b/filters/email-libravatar.lua
index b0e2447..7336baf 100644
--- a/filters/email-libravatar.lua
+++ b/filters/email-libravatar.lua
@@ -3,15 +3,24 @@
-- prefix in filters.
--
-- Requirements:
--- luacrypto >= 0.3
--- <http://mkottman.github.io/luacrypto/>
+-- luaossl
+-- <http://25thandclement.com/~william/projects/luaossl.html>
--
-local crypto = require("crypto")
+local digest = require("openssl.digest")
+
+function md5_hex(input)
+ local b = digest.new("md5"):final(input)
+ local x = ""
+ for i = 1, #b do
+ x = x .. string.format("%.2x", string.byte(b, i))
+ end
+ return x
+end
function filter_open(email, page)
buffer = ""
- md5 = crypto.digest("md5", email:sub(2, -2):lower())
+ md5 = md5_hex(email:sub(2, -2):lower())
end
function filter_close()
diff --git a/filters/file-authentication.lua b/filters/file-authentication.lua
index 6ee1e19..0248804 100644
--- a/filters/file-authentication.lua
+++ b/filters/file-authentication.lua
@@ -1,15 +1,15 @@
-- This script may be used with the auth-filter.
--
-- Requirements:
--- luacrypto >= 0.3
--- <http://mkottman.github.io/luacrypto/>
+-- luaossl
+-- <http://25thandclement.com/~william/projects/luaossl.html>
-- luaposix
-- <https://github.com/luaposix/luaposix>
--
local sysstat = require("posix.sys.stat")
local unistd = require("posix.unistd")
-local crypto = require("crypto")
-
+local rand = require("openssl.rand")
+local hmac = require("openssl.hmac")
-- This file should contain a series of lines in the form of:
-- username1:hash1
@@ -225,6 +225,13 @@ function get_cookie(cookies, name)
return url_decode(string.match(cookies, ";" .. name .. "=(.-);"))
end
+function tohex(b)
+ local x = ""
+ for i = 1, #b do
+ x = x .. string.format("%.2x", string.byte(b, i))
+ end
+ return x
+end
--
--
@@ -242,12 +249,12 @@ function get_secret()
local secret_file = io.open(secret_filename, "r")
if secret_file == nil then
local old_umask = sysstat.umask(63)
- local temporary_filename = secret_filename .. ".tmp." .. crypto.hex(crypto.rand.bytes(16))
+ local temporary_filename = secret_filename .. ".tmp." .. tohex(rand.bytes(16))
local temporary_file = io.open(temporary_filename, "w")
if temporary_file == nil then
os.exit(177)
end
- temporary_file:write(crypto.hex(crypto.rand.bytes(32)))
+ temporary_file:write(tohex(rand.bytes(32)))
temporary_file:close()
unistd.link(temporary_filename, secret_filename) -- Intentionally fails in the case that another process is doing the same.
unistd.unlink(temporary_filename)
@@ -272,7 +279,7 @@ function validate_value(expected_field, cookie)
local field = ""
local expiration = 0
local salt = ""
- local hmac = ""
+ local chmac = ""
if cookie == nil or cookie:len() < 3 or cookie:sub(1, 1) == "|" then
return nil
@@ -291,19 +298,19 @@ function validate_value(expected_field, cookie)
elseif i == 3 then
salt = component
elseif i == 4 then
- hmac = component
+ chmac = component
else
break
end
i = i + 1
end
- if hmac == nil or hmac:len() == 0 then
+ if chmac == nil or chmac:len() == 0 then
return nil
end
-- Lua hashes strings, so these comparisons are time invariant.
- if hmac ~= crypto.hmac.digest("sha256", field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt, get_secret()) then
+ if chmac ~= tohex(hmac.new(get_secret(), "sha256"):final(field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt)) then
return nil
end
@@ -324,11 +331,11 @@ function secure_value(field, value, expiration)
end
local authstr = ""
- local salt = crypto.hex(crypto.rand.bytes(16))
+ local salt = tohex(rand.bytes(16))
value = url_encode(value)
field = url_encode(field)
authstr = field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt
- authstr = authstr .. "|" .. crypto.hmac.digest("sha256", authstr, get_secret())
+ authstr = authstr .. "|" .. tohex(hmac.new(get_secret(), "sha256"):final(authstr))
return authstr
end
diff --git a/filters/gentoo-ldap-authentication.lua b/filters/gentoo-ldap-authentication.lua
index b4d98c2..673c88d 100644
--- a/filters/gentoo-ldap-authentication.lua
+++ b/filters/gentoo-ldap-authentication.lua
@@ -1,8 +1,8 @@
-- This script may be used with the auth-filter. Be sure to configure it as you wish.
--
-- Requirements:
--- luacrypto >= 0.3
--- <http://mkottman.github.io/luacrypto/>
+-- luaossl
+-- <http://25thandclement.com/~william/projects/luaossl.html>
-- lualdap >= 1.2
-- <https://git.zx2c4.com/lualdap/about/>
-- luaposix
@@ -10,9 +10,9 @@
--
local sysstat = require("posix.sys.stat")
local unistd = require("posix.unistd")
-local crypto = require("crypto")
local lualdap = require("lualdap")
-
+local rand = require("openssl.rand")
+local hmac = require("openssl.hmac")
--
--
@@ -226,6 +226,13 @@ function get_cookie(cookies, name)
return string.match(cookies, ";" .. name .. "=(.-);")
end
+function tohex(b)
+ local x = ""
+ for i = 1, #b do
+ x = x .. string.format("%.2x", string.byte(b, i))
+ end
+ return x
+end
--
--
@@ -243,12 +250,12 @@ function get_secret()
local secret_file = io.open(secret_filename, "r")
if secret_file == nil then
local old_umask = sysstat.umask(63)
- local temporary_filename = secret_filename .. ".tmp." .. crypto.hex(crypto.rand.bytes(16))
+ local temporary_filename = secret_filename .. ".tmp." .. tohex(rand.bytes(16))
local temporary_file = io.open(temporary_filename, "w")
if temporary_file == nil then
os.exit(177)
end
- temporary_file:write(crypto.hex(crypto.rand.bytes(32)))
+ temporary_file:write(tohex(rand.bytes(32)))
temporary_file:close()
unistd.link(temporary_filename, secret_filename) -- Intentionally fails in the case that another process is doing the same.
unistd.unlink(temporary_filename)
@@ -273,7 +280,7 @@ function validate_value(expected_field, cookie)
local field = ""
local expiration = 0
local salt = ""
- local hmac = ""
+ local chmac = ""
if cookie == nil or cookie:len() < 3 or cookie:sub(1, 1) == "|" then
return nil
@@ -292,19 +299,19 @@ function validate_value(expected_field, cookie)
elseif i == 3 then
salt = component
elseif i == 4 then
- hmac = component
+ chmac = component
else
break
end
i = i + 1
end
- if hmac == nil or hmac:len() == 0 then
+ if chmac == nil or chmac:len() == 0 then
return nil
end
-- Lua hashes strings, so these comparisons are time invariant.
- if hmac ~= crypto.hmac.digest("sha256", field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt, get_secret()) then
+ if chmac ~= tohex(hmac.new(get_secret(), "sha256"):final(field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt)) then
return nil
end
@@ -325,11 +332,11 @@ function secure_value(field, value, expiration)
end
local authstr = ""
- local salt = crypto.hex(crypto.rand.bytes(16))
+ local salt = tohex(rand.bytes(16))
value = url_encode(value)
field = url_encode(field)
authstr = field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt
- authstr = authstr .. "|" .. crypto.hmac.digest("sha256", authstr, get_secret())
+ authstr = authstr .. "|" .. tohex(hmac.new(get_secret(), "sha256"):final(authstr))
return authstr
end
diff --git a/filters/html-converters/md2html b/filters/html-converters/md2html
index ebf3856..59f43a8 100755
--- a/filters/html-converters/md2html
+++ b/filters/html-converters/md2html
@@ -3,6 +3,7 @@ import markdown
import sys
import io
from pygments.formatters import HtmlFormatter
+from markdown.extensions.toc import TocExtension
sys.stdin = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
sys.stdout.write('''
@@ -48,10 +49,14 @@ sys.stdout.write('''
line-height: 1;
padding-left: 0;
margin-left: -22px;
- top: 15%}
+ top: 15%;
+}
.markdown-body h1:hover a.anchor .mini-icon-link, .markdown-body h2:hover a.anchor .mini-icon-link, .markdown-body h3:hover a.anchor .mini-icon-link, .markdown-body h4:hover a.anchor .mini-icon-link, .markdown-body h5:hover a.anchor .mini-icon-link, .markdown-body h6:hover a.anchor .mini-icon-link {
display: inline-block;
}
+div#cgit .markdown-body h1 a.toclink, div#cgit .markdown-body h2 a.toclink, div#cgit .markdown-body h3 a.toclink, div#cgit .markdown-body h4 a.toclink, div#cgit .markdown-body h5 a.toclink, div#cgit .markdown-body h6 a.toclink {
+ color: black;
+}
.markdown-body h1 tt, .markdown-body h1 code, .markdown-body h2 tt, .markdown-body h2 code, .markdown-body h3 tt, .markdown-body h3 code, .markdown-body h4 tt, .markdown-body h4 code, .markdown-body h5 tt, .markdown-body h5 code, .markdown-body h6 tt, .markdown-body h6 code {
font-size: inherit;
}
@@ -81,11 +86,7 @@ sys.stdout.write('''
margin: 15px 0;
}
.markdown-body hr {
- background: transparent url("/dirty-shade.png") repeat-x 0 0;
- border: 0 none;
- color: #ccc;
- height: 4px;
- padding: 0;
+ border: 2px solid #ccc;
}
.markdown-body>h2:first-child, .markdown-body>h1:first-child, .markdown-body>h1:first-child+h2, .markdown-body>h3:first-child, .markdown-body>h4:first-child, .markdown-body>h5:first-child, .markdown-body>h6:first-child {
margin-top: 0;
@@ -290,5 +291,14 @@ sys.stdout.write('''
sys.stdout.write("<div class='markdown-body'>")
sys.stdout.flush()
# Note: you may want to run this through bleach for sanitization
-markdown.markdownFromFile(output_format="html5", extensions=["markdown.extensions.fenced_code", "markdown.extensions.codehilite", "markdown.extensions.tables"], extension_configs={"markdown.extensions.codehilite":{"css_class":"highlight"}})
+markdown.markdownFromFile(
+ output_format="html5",
+ extensions=[
+ "markdown.extensions.fenced_code",
+ "markdown.extensions.codehilite",
+ "markdown.extensions.tables",
+ "markdown.extensions.sane_lists",
+ TocExtension(anchorlink=True)],
+ extension_configs={
+ "markdown.extensions.codehilite":{"css_class":"highlight"}})
sys.stdout.write("</div>")
diff --git a/filters/simple-authentication.lua b/filters/simple-authentication.lua
index 77d1fd0..23d3457 100644
--- a/filters/simple-authentication.lua
+++ b/filters/simple-authentication.lua
@@ -1,15 +1,15 @@
-- This script may be used with the auth-filter. Be sure to configure it as you wish.
--
-- Requirements:
--- luacrypto >= 0.3
--- <http://mkottman.github.io/luacrypto/>
+-- luaossl
+-- <http://25thandclement.com/~william/projects/luaossl.html>
-- luaposix
-- <https://github.com/luaposix/luaposix>
--
local sysstat = require("posix.sys.stat")
local unistd = require("posix.unistd")
-local crypto = require("crypto")
-
+local rand = require("openssl.rand")
+local hmac = require("openssl.hmac")
--
--
@@ -180,6 +180,13 @@ function get_cookie(cookies, name)
return url_decode(string.match(cookies, ";" .. name .. "=(.-);"))
end
+function tohex(b)
+ local x = ""
+ for i = 1, #b do
+ x = x .. string.format("%.2x", string.byte(b, i))
+ end
+ return x
+end
--
--
@@ -197,12 +204,12 @@ function get_secret()
local secret_file = io.open(secret_filename, "r")
if secret_file == nil then
local old_umask = sysstat.umask(63)
- local temporary_filename = secret_filename .. ".tmp." .. crypto.hex(crypto.rand.bytes(16))
+ local temporary_filename = secret_filename .. ".tmp." .. tohex(rand.bytes(16))
local temporary_file = io.open(temporary_filename, "w")
if temporary_file == nil then
os.exit(177)
end
- temporary_file:write(crypto.hex(crypto.rand.bytes(32)))
+ temporary_file:write(tohex(rand.bytes(32)))
temporary_file:close()
unistd.link(temporary_filename, secret_filename) -- Intentionally fails in the case that another process is doing the same.
unistd.unlink(temporary_filename)
@@ -227,7 +234,7 @@ function validate_value(expected_field, cookie)
local field = ""
local expiration = 0
local salt = ""
- local hmac = ""
+ local chmac = ""
if cookie == nil or cookie:len() < 3 or cookie:sub(1, 1) == "|" then
return nil
@@ -246,19 +253,19 @@ function validate_value(expected_field, cookie)
elseif i == 3 then
salt = component
elseif i == 4 then
- hmac = component
+ chmac = component
else
break
end
i = i + 1
end
- if hmac == nil or hmac:len() == 0 then
+ if chmac == nil or chmac:len() == 0 then
return nil
end
-- Lua hashes strings, so these comparisons are time invariant.
- if hmac ~= crypto.hmac.digest("sha256", field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt, get_secret()) then
+ if chmac ~= tohex(hmac.new(get_secret(), "sha256"):final(field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt)) then
return nil
end
@@ -279,11 +286,11 @@ function secure_value(field, value, expiration)
end
local authstr = ""
- local salt = crypto.hex(crypto.rand.bytes(16))
+ local salt = tohex(rand.bytes(16))
value = url_encode(value)
field = url_encode(field)
authstr = field .. "|" .. value .. "|" .. tostring(expiration) .. "|" .. salt
- authstr = authstr .. "|" .. crypto.hmac.digest("sha256", authstr, get_secret())
+ authstr = authstr .. "|" .. tohex(hmac.new(get_secret(), "sha256"):final(authstr))
return authstr
end
diff --git a/git b/git
-Subproject 53f9a3e157dbbc901a02ac2c73346d375e24978
+Subproject ebf3c04b262aa27fbb97f8a0156c2347fecafaf
diff --git a/parsing.c b/parsing.c
index 12453c2..72b59b3 100644
--- a/parsing.c
+++ b/parsing.c
@@ -63,8 +63,7 @@ static char *substr(const char *head, const char *tail)
if (tail < head)
return xstrdup("");
buf = xmalloc(tail - head + 1);
- strncpy(buf, head, tail - head);
- buf[tail - head] = '\0';
+ strlcpy(buf, head, tail - head + 1);
return buf;
}
@@ -78,7 +77,7 @@ static void parse_user(const char *t, char **name, char **email, unsigned long *
email_len = ident.mail_end - ident.mail_begin;
*email = xmalloc(strlen("<") + email_len + strlen(">") + 1);
- sprintf(*email, "<%.*s>", email_len, ident.mail_begin);
+ xsnprintf(*email, email_len + 3, "<%.*s>", email_len, ident.mail_begin);
if (ident.date_begin)
*date = strtoul(ident.date_begin, NULL, 10);
@@ -128,9 +127,8 @@ static int end_of_header(const char *p)
struct commitinfo *cgit_parse_commit(struct commit *commit)
{
- const int sha1hex_len = 40;
struct commitinfo *ret;
- const char *p = get_cached_commit_buffer(commit, NULL);
+ const char *p = repo_get_commit_buffer(the_repository, commit, NULL);
const char *t;
ret = xcalloc(1, sizeof(struct commitinfo));
@@ -141,10 +139,10 @@ struct commitinfo *cgit_parse_commit(struct commit *commit)
if (!skip_prefix(p, "tree ", &p))
die("Bad commit: %s", oid_to_hex(&commit->object.oid));
- p += sha1hex_len + 1;
+ p += the_hash_algo->hexsz + 1;
while (skip_prefix(p, "parent ", &p))
- p += sha1hex_len + 1;
+ p += the_hash_algo->hexsz + 1;
if (p && skip_prefix(p, "author ", &p)) {
parse_user(p, &ret->author, &ret->author_email,
diff --git a/shared.c b/shared.c
index 609bd2a..8115469 100644
--- a/shared.c
+++ b/shared.c
@@ -58,6 +58,7 @@ struct cgit_repo *cgit_add_repo(const char *url)
ret->homepage = NULL;
ret->section = ctx.cfg.section;
ret->snapshots = ctx.cfg.snapshots;
+ ret->enable_blame = ctx.cfg.enable_blame;
ret->enable_commit_graph = ctx.cfg.enable_commit_graph;
ret->enable_log_filecount = ctx.cfg.enable_log_filecount;
ret->enable_log_linecount = ctx.cfg.enable_log_linecount;
@@ -161,7 +162,7 @@ static struct refinfo *cgit_mk_refinfo(const char *refname, const struct object_
ref = xmalloc(sizeof (struct refinfo));
ref->refname = xstrdup(refname);
- ref->object = parse_object(oid);
+ ref->object = parse_object(the_repository, oid);
switch (ref->object->type) {
case OBJ_TAG:
ref->tag = cgit_parse_tag((struct tag *)ref->object);
@@ -325,7 +326,7 @@ int cgit_diff_files(const struct object_id *old_oid,
diff_params.flags |= XDF_IGNORE_WHITESPACE;
emit_params.ctxlen = context > 0 ? context : 3;
emit_params.flags = XDL_EMIT_FUNCNAMES;
- emit_cb.outf = filediff_cb;
+ emit_cb.out_line = filediff_cb;
emit_cb.priv = fn;
xdl_diff(&file1, &file2, &diff_params, &emit_params, &emit_cb);
if (file1.size)
diff --git a/tests/setup.sh b/tests/setup.sh
index 7590f04..8db810f 100755
--- a/tests/setup.sh
+++ b/tests/setup.sh
@@ -80,13 +80,17 @@ mkrepo() {
git commit -m "commit $n"
n=$(expr $n + 1)
done
- if test "$3" = "testplus"
- then
+ case "$3" in
+ testplus)
echo "hello" >a+b
git add a+b
git commit -m "add a+b"
git branch "1+2"
- fi
+ ;;
+ commit-graph)
+ git commit-graph write
+ ;;
+ esac
)
}
@@ -95,7 +99,7 @@ setup_repos()
rm -rf cache
mkdir -p cache
mkrepo repos/foo 5 >/dev/null
- mkrepo repos/bar 50 >/dev/null
+ mkrepo repos/bar 50 commit-graph >/dev/null
mkrepo repos/foo+bar 10 testplus >/dev/null
mkrepo "repos/with space" 2 >/dev/null
mkrepo repos/filter 5 testplus >/dev/null
@@ -104,7 +108,7 @@ virtual-root=/
cache-root=$PWD/cache
cache-size=1021
-snapshots=tar.gz tar.bz zip
+snapshots=tar.gz tar.bz tar.lz tar.xz tar.zst zip
enable-log-filecount=1
enable-log-linecount=1
summary-log=5
diff --git a/tests/t0001-validate-git-versions.sh b/tests/t0001-validate-git-versions.sh
index a65b35e..dd84fe3 100755
--- a/tests/t0001-validate-git-versions.sh
+++ b/tests/t0001-validate-git-versions.sh
@@ -1,5 +1,9 @@
#!/bin/sh
+if [ "${CGIT_TEST_NO_GIT_VERSION}" = "YesPlease" ]; then
+ exit 0
+fi
+
test_description='Check Git version is correct'
CGIT_TEST_NO_CREATE_REPOS=YesPlease
. ./setup.sh
@@ -29,11 +33,11 @@ test_expect_success 'test submodule version matches Makefile' '
else
(
cd ../.. &&
- sm_sha1=$(git ls-files --stage -- git |
+ sm_oid=$(git ls-files --stage -- git |
sed -e "s/^[0-9]* \\([0-9a-f]*\\) [0-9] .*$/\\1/") &&
cd git &&
- git describe --match "v[0-9]*" $sm_sha1
- ) | sed -e "s/^v//" >sm_version &&
+ git describe --match "v[0-9]*" $sm_oid
+ ) | sed -e "s/^v//" -e "s/-/./" >sm_version &&
test_cmp sm_version makefile_version
fi
'
diff --git a/tests/t0105-commit.sh b/tests/t0105-commit.sh
index 9cdf55c..1a12ee3 100755
--- a/tests/t0105-commit.sh
+++ b/tests/t0105-commit.sh
@@ -25,7 +25,7 @@ test_expect_success 'get root commit' '
'
test_expect_success 'root commit contains diffstat' '
- grep "<a href=./foo/diff/file-1.id=[0-9a-f]\{40\}.>file-1</a>" tmp
+ grep "<a href=./foo/diff/file-1.id=[0-9a-f]\{40,64\}.>file-1</a>" tmp
'
test_expect_success 'root commit contains diff' '
diff --git a/tests/t0107-snapshot.sh b/tests/t0107-snapshot.sh
index 6cf7aaa..0811ec4 100755
--- a/tests/t0107-snapshot.sh
+++ b/tests/t0107-snapshot.sh
@@ -25,7 +25,7 @@ test_expect_success 'verify gzip format' '
test_expect_success 'untar' '
rm -rf master &&
- tar -xzf master.tar.gz
+ gzip -dc master.tar.gz | tar -xf -
'
test_expect_success 'count files' '
@@ -38,6 +38,129 @@ test_expect_success 'verify untarred file-5' '
test_line_count = 1 master/file-5
'
+if test -n "$(which lzip 2>/dev/null)"; then
+ test_set_prereq LZIP
+else
+ say 'Skipping LZIP validation tests: lzip not found'
+fi
+
+test_expect_success LZIP 'get foo/snapshot/master.tar.lz' '
+ cgit_url "foo/snapshot/master.tar.lz" >tmp
+'
+
+test_expect_success LZIP 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-lzip" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.lz."
+'
+
+test_expect_success LZIP 'strip off the header lines' '
+ strip_headers <tmp >master.tar.lz
+'
+
+test_expect_success LZIP 'verify lzip format' '
+ lzip --test master.tar.lz
+'
+
+test_expect_success LZIP 'untar' '
+ rm -rf master &&
+ lzip -dc master.tar.lz | tar -xf -
+'
+
+test_expect_success LZIP 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success LZIP 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
+if test -n "$(which xz 2>/dev/null)"; then
+ test_set_prereq XZ
+else
+ say 'Skipping XZ validation tests: xz not found'
+fi
+
+test_expect_success XZ 'get foo/snapshot/master.tar.xz' '
+ cgit_url "foo/snapshot/master.tar.xz" >tmp
+'
+
+test_expect_success XZ 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-xz" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.xz."
+'
+
+test_expect_success XZ 'strip off the header lines' '
+ strip_headers <tmp >master.tar.xz
+'
+
+test_expect_success XZ 'verify xz format' '
+ xz --test master.tar.xz
+'
+
+test_expect_success XZ 'untar' '
+ rm -rf master &&
+ xz -dc master.tar.xz | tar -xf -
+'
+
+test_expect_success XZ 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success XZ 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
+if test -n "$(which zstd 2>/dev/null)"; then
+ test_set_prereq ZSTD
+else
+ say 'Skipping ZSTD validation tests: zstd not found'
+fi
+
+test_expect_success ZSTD 'get foo/snapshot/master.tar.zst' '
+ cgit_url "foo/snapshot/master.tar.zst" >tmp
+'
+
+test_expect_success ZSTD 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-zstd" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.zst."
+'
+
+test_expect_success ZSTD 'strip off the header lines' '
+ strip_headers <tmp >master.tar.zst
+'
+
+test_expect_success ZSTD 'verify zstd format' '
+ zstd --test master.tar.zst
+'
+
+test_expect_success ZSTD 'untar' '
+ rm -rf master &&
+ zstd -dc master.tar.zst | tar -xf -
+'
+
+test_expect_success ZSTD 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success ZSTD 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
test_expect_success 'get foo/snapshot/master.zip' '
cgit_url "foo/snapshot/master.zip" >tmp
'
diff --git a/tests/t0109-gitconfig.sh b/tests/t0109-gitconfig.sh
index 3ba6684..189ef28 100755
--- a/tests/t0109-gitconfig.sh
+++ b/tests/t0109-gitconfig.sh
@@ -9,6 +9,12 @@ test -n "$(which strace 2>/dev/null)" || {
exit
}
+strace true 2>/dev/null || {
+ skip_all='Skipping access validation tests: strace not functional'
+ test_done
+ exit
+}
+
test_no_home_access () {
non_existent_path="/path/to/some/place/that/does/not/possibly/exist"
while test -d "$non_existent_path"; do
@@ -19,7 +25,7 @@ test_no_home_access () {
-E CGIT_CONFIG="$PWD/cgitrc" \
-E QUERY_STRING="url=$1" \
-e access -f -o strace.out cgit &&
- test_must_fail grep "$non_existent_path" strace.out
+ ! grep "$non_existent_path" strace.out
}
test_no_home_access_success() {
diff --git a/ui-atom.c b/ui-atom.c
index 3866823..1056f36 100644
--- a/ui-atom.c
+++ b/ui-atom.c
@@ -83,7 +83,7 @@ static void add_entry(struct commit *commit, const char *host)
}
-void cgit_print_atom(char *tip, char *path, int max_count)
+void cgit_print_atom(char *tip, const char *path, int max_count)
{
char *host;
const char *argv[] = {NULL, tip, NULL, NULL, NULL};
@@ -140,7 +140,7 @@ void cgit_print_atom(char *tip, char *path, int max_count)
}
while ((commit = get_revision(&rev)) != NULL) {
add_entry(commit, host);
- free_commit_buffer(commit);
+ free_commit_buffer(the_repository->parsed_objects, commit);
free_commit_list(commit->parents);
commit->parents = NULL;
}
diff --git a/ui-atom.h b/ui-atom.h
index 749ffd3..dda953b 100644
--- a/ui-atom.h
+++ b/ui-atom.h
@@ -1,6 +1,6 @@
#ifndef UI_ATOM_H
#define UI_ATOM_H
-extern void cgit_print_atom(char *tip, char *path, int max_count);
+extern void cgit_print_atom(char *tip, const char *path, int max_count);
#endif
diff --git a/ui-blame.c b/ui-blame.c
index 50d0580..03136f7 100644
--- a/ui-blame.c
+++ b/ui-blame.c
@@ -10,7 +10,7 @@
#include "ui-blame.h"
#include "html.h"
#include "ui-shared.h"
-#include "argv-array.h"
+#include "strvec.h"
#include "blame.h"
@@ -48,7 +48,7 @@ static void emit_blame_entry_hash(struct blame_entry *ent)
unsigned long line = 0;
char *detail = emit_suspect_detail(suspect);
- html("<span class='sha1'>");
+ html("<span class='oid'>");
cgit_commit_link(find_unique_abbrev(oid, DEFAULT_ABBREV), detail,
NULL, ctx.qry.head, oid_to_hex(oid), suspect->path);
html("</span>");
@@ -104,7 +104,7 @@ static void print_object(const struct object_id *oid, const char *path,
enum object_type type;
char *buf;
unsigned long size;
- struct argv_array rev_argv = ARGV_ARRAY_INIT;
+ struct strvec rev_argv = STRVEC_INIT;
struct rev_info revs;
struct blame_scoreboard sb;
struct blame_origin *o;
@@ -124,14 +124,16 @@ static void print_object(const struct object_id *oid, const char *path,
return;
}
- argv_array_push(&rev_argv, "blame");
- argv_array_push(&rev_argv, rev);
+ strvec_push(&rev_argv, "blame");
+ strvec_push(&rev_argv, rev);
init_revisions(&revs, NULL);
revs.diffopt.flags.allow_textconv = 1;
- setup_revisions(rev_argv.argc, rev_argv.argv, &revs, NULL);
+ setup_revisions(rev_argv.nr, rev_argv.v, &revs, NULL);
init_scoreboard(&sb);
sb.revs = &revs;
- setup_scoreboard(&sb, path, &o);
+ sb.repo = the_repository;
+ sb.path = path;
+ setup_scoreboard(&sb, &o);
o->suspects = blame_entry_prepend(NULL, 0, sb.num_lines, o);
prio_queue_put(&sb.commits, o->commit);
blame_origin_decref(o);
@@ -219,8 +221,7 @@ cleanup:
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage,
- void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -255,7 +256,7 @@ static int basedir_len(const char *path)
void cgit_print_blame(void)
{
- const char *rev = ctx.qry.sha1;
+ const char *rev = ctx.qry.oid;
struct object_id oid;
struct commit *commit;
struct pathspec_item path_items = {
@@ -278,7 +279,7 @@ void cgit_print_blame(void)
"Invalid revision name: %s", rev);
return;
}
- commit = lookup_commit_reference(&oid);
+ commit = lookup_commit_reference(the_repository, &oid);
if (!commit || parse_commit(commit)) {
cgit_print_error_page(404, "Not found",
"Invalid commit reference: %s", rev);
@@ -289,8 +290,8 @@ void cgit_print_blame(void)
walk_tree_ctx.match_baselen = (path_items.match) ?
basedir_len(path_items.match) : -1;
- read_tree_recursive(commit->maybe_tree, "", 0, 0, &paths, walk_tree,
- &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.state)
cgit_print_error_page(404, "Not found", "Not found");
else if (walk_tree_ctx.state == 2)
diff --git a/ui-blob.c b/ui-blob.c
index 7b6da2a..c10ae42 100644
--- a/ui-blob.c
+++ b/ui-blob.c
@@ -19,7 +19,7 @@ struct walk_tree_context {
};
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -56,7 +56,9 @@ int cgit_ref_path_exists(const char *path, const char *ref, int file_only)
goto done;
if (oid_object_info(the_repository, &oid, &size) != OBJ_COMMIT)
goto done;
- read_tree_recursive(lookup_commit_reference(&oid)->maybe_tree, "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository,
+ repo_get_commit_tree(the_repository, lookup_commit_reference(the_repository, &oid)),
+ &paths, walk_tree, &walk_tree_ctx);
done:
free(path_items.match);
@@ -89,8 +91,9 @@ int cgit_print_file(char *path, const char *head, int file_only)
return -1;
type = oid_object_info(the_repository, &oid, &size);
if (type == OBJ_COMMIT) {
- commit = lookup_commit_reference(&oid);
- read_tree_recursive(commit->maybe_tree, "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ commit = lookup_commit_reference(the_repository, &oid);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.found_path)
return -1;
type = oid_object_info(the_repository, &oid, &size);
@@ -145,8 +148,9 @@ void cgit_print_blob(const char *hex, char *path, const char *head, int file_onl
type = oid_object_info(the_repository, &oid, &size);
if ((!hex) && type == OBJ_COMMIT && path) {
- commit = lookup_commit_reference(&oid);
- read_tree_recursive(commit->maybe_tree, "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ commit = lookup_commit_reference(the_repository, &oid);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
type = oid_object_info(the_repository, &oid, &size);
}
diff --git a/ui-clone.c b/ui-clone.c
index 6ba8f36..5dccb63 100644
--- a/ui-clone.c
+++ b/ui-clone.c
@@ -19,12 +19,12 @@ static int print_ref_info(const char *refname, const struct object_id *oid,
{
struct object *obj;
- if (!(obj = parse_object(oid)))
+ if (!(obj = parse_object(the_repository, oid)))
return 0;
htmlf("%s\t%s\n", oid_to_hex(oid), refname);
if (obj->type == OBJ_TAG) {
- if (!(obj = deref_tag(obj, refname, 0)))
+ if (!(obj = deref_tag(the_repository, obj, refname, 0)))
return 0;
htmlf("%s\t%s^{}\n", oid_to_hex(&obj->oid), refname);
}
diff --git a/ui-commit.c b/ui-commit.c
index 995cb93..948118c 100644
--- a/ui-commit.c
+++ b/ui-commit.c
@@ -31,7 +31,7 @@ void cgit_print_commit(char *hex, const char *prefix)
"Bad object id: %s", hex);
return;
}
- commit = lookup_commit_reference(&oid);
+ commit = lookup_commit_reference(the_repository, &oid);
if (!commit) {
cgit_print_error_page(404, "Not found",
"Bad commit reference: %s", hex);
@@ -70,15 +70,15 @@ void cgit_print_commit(char *hex, const char *prefix)
html_txt(show_date(info->committer_date, info->committer_tz,
cgit_date_mode(DATE_ISO8601)));
html("</td></tr>\n");
- html("<tr><th>commit</th><td colspan='2' class='sha1'>");
+ html("<tr><th>commit</th><td colspan='2' class='oid'>");
tmp = oid_to_hex(&commit->object.oid);
cgit_commit_link(tmp, NULL, NULL, ctx.qry.head, tmp, prefix);
html(" (");
cgit_patch_link("patch", NULL, NULL, NULL, tmp, prefix);
html(")</td></tr>\n");
- html("<tr><th>tree</th><td colspan='2' class='sha1'>");
+ html("<tr><th>tree</th><td colspan='2' class='oid'>");
tmp = xstrdup(hex);
- cgit_tree_link(oid_to_hex(&commit->maybe_tree->object.oid), NULL, NULL,
+ cgit_tree_link(oid_to_hex(get_commit_tree_oid(commit)), NULL, NULL,
ctx.qry.head, tmp, NULL);
if (prefix) {
html(" /");
@@ -87,7 +87,7 @@ void cgit_print_commit(char *hex, const char *prefix)
free(tmp);
html("</td></tr>\n");
for (p = commit->parents; p; p = p->next) {
- parent = lookup_commit_reference(&p->item->object.oid);
+ parent = lookup_commit_reference(the_repository, &p->item->object.oid);
if (!parent) {
html("<tr><td colspan='3'>");
cgit_print_error("Error reading parent commit");
@@ -95,7 +95,7 @@ void cgit_print_commit(char *hex, const char *prefix)
continue;
}
html("<tr><th>parent</th>"
- "<td colspan='2' class='sha1'>");
+ "<td colspan='2' class='oid'>");
tmp = tmp2 = oid_to_hex(&p->item->object.oid);
if (ctx.repo->enable_subject_links) {
parent_info = cgit_parse_commit(parent);
@@ -109,7 +109,7 @@ void cgit_print_commit(char *hex, const char *prefix)
parents++;
}
if (ctx.repo->snapshots) {
- html("<tr><th>download</th><td colspan='2' class='sha1'>");
+ html("<tr><th>download</th><td colspan='2' class='oid'>");
cgit_print_snapshot_links(ctx.repo, hex, "<br/>");
html("</td></tr>");
}
@@ -139,7 +139,7 @@ void cgit_print_commit(char *hex, const char *prefix)
tmp = oid_to_hex(&commit->parents->item->object.oid);
else
tmp = NULL;
- cgit_print_diff(ctx.qry.sha1, tmp, prefix, 0, 0);
+ cgit_print_diff(ctx.qry.oid, tmp, prefix, 0, 0);
}
strbuf_release(&notes);
cgit_free_commitinfo(info);
diff --git a/ui-diff.c b/ui-diff.c
index e33e9fb..5ed5990 100644
--- a/ui-diff.c
+++ b/ui-diff.c
@@ -82,7 +82,7 @@ static void print_fileinfo(struct fileinfo *info)
}
html("<tr>");
- htmlf("<td class='mode'>");
+ html("<td class='mode'>");
if (is_null_oid(info->new_oid)) {
cgit_print_filemode(info->old_mode);
} else {
@@ -97,8 +97,8 @@ static void print_fileinfo(struct fileinfo *info)
html("]</span>");
}
htmlf("</td><td class='%s'>", class);
- cgit_diff_link(info->new_path, NULL, NULL, ctx.qry.head, ctx.qry.sha1,
- ctx.qry.sha2, info->new_path);
+ cgit_diff_link(info->new_path, NULL, NULL, ctx.qry.head, ctx.qry.oid,
+ ctx.qry.oid2, info->new_path);
if (info->status == DIFF_STATUS_COPIED || info->status == DIFF_STATUS_RENAMED) {
htmlf(" (%s from ",
info->status == DIFF_STATUS_COPIED ? "copied" : "renamed");
@@ -194,8 +194,8 @@ static void cgit_print_diffstat(const struct object_id *old_oid,
int i;
html("<div class='diffstat-header'>");
- cgit_diff_link("Diffstat", NULL, NULL, ctx.qry.head, ctx.qry.sha1,
- ctx.qry.sha2, NULL);
+ cgit_diff_link("Diffstat", NULL, NULL, ctx.qry.head, ctx.qry.oid,
+ ctx.qry.oid2, NULL);
if (prefix) {
html(" (limited to '");
html_txt(prefix);
@@ -407,13 +407,13 @@ void cgit_print_diff(const char *new_rev, const char *old_rev,
"Bad object name: %s", new_rev);
return;
}
- commit = lookup_commit_reference(new_rev_oid);
+ commit = lookup_commit_reference(the_repository, new_rev_oid);
if (!commit || parse_commit(commit)) {
cgit_print_error_page(404, "Not found",
"Bad commit: %s", oid_to_hex(new_rev_oid));
return;
}
- new_tree_oid = &commit->maybe_tree->object.oid;
+ new_tree_oid = get_commit_tree_oid(commit);
if (old_rev) {
if (get_oid(old_rev, old_rev_oid)) {
@@ -428,13 +428,13 @@ void cgit_print_diff(const char *new_rev, const char *old_rev,
}
if (!is_null_oid(old_rev_oid)) {
- commit2 = lookup_commit_reference(old_rev_oid);
+ commit2 = lookup_commit_reference(the_repository, old_rev_oid);
if (!commit2 || parse_commit(commit2)) {
cgit_print_error_page(404, "Not found",
"Bad commit: %s", oid_to_hex(old_rev_oid));
return;
}
- old_tree_oid = &commit2->maybe_tree->object.oid;
+ old_tree_oid = get_commit_tree_oid(commit2);
} else {
old_tree_oid = NULL;
}
diff --git a/ui-log.c b/ui-log.c
index d696e20..20774bf 100644
--- a/ui-log.c
+++ b/ui-log.c
@@ -10,7 +10,7 @@
#include "ui-log.h"
#include "html.h"
#include "ui-shared.h"
-#include "argv-array.h"
+#include "strvec.h"
static int files, add_lines, rem_lines, lines_counted;
@@ -65,9 +65,10 @@ void show_commit_decorations(struct commit *commit)
return;
html("<span class='decoration'>");
while (deco) {
- struct object_id peeled;
+ struct object_id oid_tag, peeled;
int is_annotated = 0;
- strncpy(buf, prettify_refname(deco->name), sizeof(buf) - 1);
+
+ strlcpy(buf, prettify_refname(deco->name), sizeof(buf));
switch(deco->type) {
case DECORATION_NONE:
/* If the git-core doesn't recognize it,
@@ -79,8 +80,8 @@ void show_commit_decorations(struct commit *commit)
ctx.qry.showmsg, 0);
break;
case DECORATION_REF_TAG:
- if (!peel_ref(deco->name, &peeled))
- is_annotated = !oidcmp(&commit->object.oid, &peeled);
+ if (!read_ref(deco->name, &oid_tag) && !peel_iterated_oid(&oid_tag, &peeled))
+ is_annotated = !oideq(&oid_tag, &peeled);
cgit_tag_link(buf, NULL, is_annotated ? "tag-annotated-deco" : "tag-deco", buf);
break;
case DECORATION_REF_REMOTE:
@@ -153,8 +154,8 @@ static int show_commit(struct commit *commit, struct rev_info *revs)
rem_lines = 0;
revs->diffopt.flags.recursive = 1;
- diff_tree_oid(&parent->maybe_tree->object.oid,
- &commit->maybe_tree->object.oid,
+ diff_tree_oid(get_commit_tree_oid(parent),
+ get_commit_tree_oid(commit),
"", &revs->diffopt);
diffcore_std(&revs->diffopt);
@@ -234,7 +235,7 @@ static void print_commit(struct commit *commit, struct rev_info *revs)
strbuf_add(&msgbuf, "\n\n", 2);
/* Place wrap_symbol at position i in info->subject */
- strcpy(info->subject + i, wrap_symbol);
+ strlcpy(info->subject + i, wrap_symbol, subject_len - i + 1);
}
}
cgit_commit_link(info->subject, NULL, NULL, ctx.qry.head,
@@ -362,27 +363,27 @@ static char *next_token(char **src)
}
void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern,
- char *path, int pager, int commit_graph, int commit_sort)
+ const char *path, int pager, int commit_graph, int commit_sort)
{
struct rev_info rev;
struct commit *commit;
- struct argv_array rev_argv = ARGV_ARRAY_INIT;
+ struct strvec rev_argv = STRVEC_INIT;
int i, columns = commit_graph ? 4 : 3;
int must_free_tip = 0;
/* rev_argv.argv[0] will be ignored by setup_revisions */
- argv_array_push(&rev_argv, "log_rev_setup");
+ strvec_push(&rev_argv, "log_rev_setup");
if (!tip)
tip = ctx.qry.head;
tip = disambiguate_ref(tip, &must_free_tip);
- argv_array_push(&rev_argv, tip);
+ strvec_push(&rev_argv, tip);
if (grep && pattern && *pattern) {
pattern = xstrdup(pattern);
if (!strcmp(grep, "grep") || !strcmp(grep, "author") ||
!strcmp(grep, "committer")) {
- argv_array_pushf(&rev_argv, "--%s=%s", grep, pattern);
+ strvec_pushf(&rev_argv, "--%s=%s", grep, pattern);
} else if (!strcmp(grep, "range")) {
char *arg;
/* Split the pattern at whitespace and add each token
@@ -390,14 +391,14 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
* rev-list options. Also, replace the previously
* pushed tip (it's no longer relevant).
*/
- argv_array_pop(&rev_argv);
+ strvec_pop(&rev_argv);
while ((arg = next_token(&pattern))) {
if (*arg == '-') {
fprintf(stderr, "Bad range expr: %s\n",
arg);
break;
}
- argv_array_push(&rev_argv, arg);
+ strvec_push(&rev_argv, arg);
}
}
}
@@ -412,22 +413,22 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
}
if (commit_graph && !ctx.qry.follow) {
- argv_array_push(&rev_argv, "--graph");
- argv_array_push(&rev_argv, "--color");
+ strvec_push(&rev_argv, "--graph");
+ strvec_push(&rev_argv, "--color");
graph_set_column_colors(column_colors_html,
COLUMN_COLORS_HTML_MAX);
}
if (commit_sort == 1)
- argv_array_push(&rev_argv, "--date-order");
+ strvec_push(&rev_argv, "--date-order");
else if (commit_sort == 2)
- argv_array_push(&rev_argv, "--topo-order");
+ strvec_push(&rev_argv, "--topo-order");
if (path && ctx.qry.follow)
- argv_array_push(&rev_argv, "--follow");
- argv_array_push(&rev_argv, "--");
+ strvec_push(&rev_argv, "--follow");
+ strvec_push(&rev_argv, "--");
if (path)
- argv_array_push(&rev_argv, path);
+ strvec_push(&rev_argv, path);
init_revisions(&rev, NULL);
rev.abbrev = DEFAULT_ABBREV;
@@ -436,7 +437,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
rev.show_root_diff = 0;
rev.ignore_missing = 1;
rev.simplify_history = 1;
- setup_revisions(rev_argv.argc, rev_argv.argv, &rev, NULL);
+ setup_revisions(rev_argv.nr, rev_argv.v, &rev, NULL);
load_ref_decorations(NULL, DECORATE_FULL_REFS);
rev.show_decorations = 1;
rev.grep_filter.ignore_case = 1;
@@ -463,7 +464,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if (pager) {
html(" (");
cgit_log_link(ctx.qry.showmsg ? "Collapse" : "Expand", NULL,
- NULL, ctx.qry.head, ctx.qry.sha1,
+ NULL, ctx.qry.head, ctx.qry.oid,
ctx.qry.vpath, ctx.qry.ofs, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg ? 0 : 1,
ctx.qry.follow);
@@ -488,7 +489,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
for (i = 0; i < ofs && (commit = get_revision(&rev)) != NULL; /* nop */) {
if (show_commit(commit, &rev))
i++;
- free_commit_buffer(commit);
+ free_commit_buffer(the_repository->parsed_objects, commit);
free_commit_list(commit->parents);
commit->parents = NULL;
}
@@ -510,7 +511,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
i++;
print_commit(commit, &rev);
}
- free_commit_buffer(commit);
+ free_commit_buffer(the_repository->parsed_objects, commit);
free_commit_list(commit->parents);
commit->parents = NULL;
}
@@ -519,7 +520,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if (ofs > 0) {
html("<li>");
cgit_log_link("[prev]", NULL, NULL, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath,
+ ctx.qry.oid, ctx.qry.vpath,
ofs - cnt, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg,
ctx.qry.follow);
@@ -528,7 +529,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if ((commit = get_revision(&rev)) != NULL) {
html("<li>");
cgit_log_link("[next]", NULL, NULL, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath,
+ ctx.qry.oid, ctx.qry.vpath,
ofs + cnt, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg,
ctx.qry.follow);
diff --git a/ui-log.h b/ui-log.h
index d324c92..325607c 100644
--- a/ui-log.h
+++ b/ui-log.h
@@ -2,7 +2,7 @@
#define UI_LOG_H
extern void cgit_print_log(const char *tip, int ofs, int cnt, char *grep,
- char *pattern, char *path, int pager,
+ char *pattern, const char *path, int pager,
int commit_graph, int commit_sort);
extern void show_commit_decorations(struct commit *commit);
diff --git a/ui-patch.c b/ui-patch.c
index 8007a11..4ac03cb 100644
--- a/ui-patch.c
+++ b/ui-patch.c
@@ -11,13 +11,16 @@
#include "html.h"
#include "ui-shared.h"
+/* two commit hashes with two dots in between and termination */
+#define REV_RANGE_LEN 2 * GIT_MAX_HEXSZ + 3
+
void cgit_print_patch(const char *new_rev, const char *old_rev,
const char *prefix)
{
struct rev_info rev;
struct commit *commit;
struct object_id new_rev_oid, old_rev_oid;
- char rev_range[2 * 40 + 3];
+ char rev_range[REV_RANGE_LEN];
const char *rev_argv[] = { NULL, "--reverse", "--format=email", rev_range, "--", prefix, NULL };
int rev_argc = ARRAY_SIZE(rev_argv) - 1;
char *patchname;
@@ -33,7 +36,7 @@ void cgit_print_patch(const char *new_rev, const char *old_rev,
"Bad object id: %s", new_rev);
return;
}
- commit = lookup_commit_reference(&new_rev_oid);
+ commit = lookup_commit_reference(the_repository, &new_rev_oid);
if (!commit) {
cgit_print_error_page(404, "Not found",
"Bad commit reference: %s", new_rev);
@@ -46,7 +49,7 @@ void cgit_print_patch(const char *new_rev, const char *old_rev,
"Bad object id: %s", old_rev);
return;
}
- if (!lookup_commit_reference(&old_rev_oid)) {
+ if (!lookup_commit_reference(the_repository, &old_rev_oid)) {
cgit_print_error_page(404, "Not found",
"Bad commit reference: %s", old_rev);
return;
@@ -58,9 +61,9 @@ void cgit_print_patch(const char *new_rev, const char *old_rev,
}
if (is_null_oid(&old_rev_oid)) {
- memcpy(rev_range, oid_to_hex(&new_rev_oid), GIT_SHA1_HEXSZ + 1);
+ memcpy(rev_range, oid_to_hex(&new_rev_oid), the_hash_algo->hexsz + 1);
} else {
- sprintf(rev_range, "%s..%s", oid_to_hex(&old_rev_oid),
+ xsnprintf(rev_range, REV_RANGE_LEN, "%s..%s", oid_to_hex(&old_rev_oid),
oid_to_hex(&new_rev_oid));
}
diff --git a/ui-plain.c b/ui-plain.c
index ddb3e48..65a205f 100644
--- a/ui-plain.c
+++ b/ui-plain.c
@@ -99,7 +99,7 @@ static void print_dir(const struct object_id *oid, const char *base,
fullpath = NULL;
}
html("<li>");
- cgit_plain_link("../", NULL, NULL, ctx.qry.head, ctx.qry.sha1,
+ cgit_plain_link("../", NULL, NULL, ctx.qry.head, ctx.qry.oid,
fullpath);
html("</li>\n");
}
@@ -118,7 +118,7 @@ static void print_dir_entry(const struct object_id *oid, const char *base,
if (S_ISGITLINK(mode)) {
cgit_submodule_link(NULL, fullpath, oid_to_hex(oid));
} else
- cgit_plain_link(path, NULL, NULL, ctx.qry.head, ctx.qry.sha1,
+ cgit_plain_link(path, NULL, NULL, ctx.qry.head, ctx.qry.oid,
fullpath);
html("</li>\n");
free(fullpath);
@@ -130,7 +130,7 @@ static void print_dir_tail(void)
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -163,7 +163,7 @@ static int basedir_len(const char *path)
void cgit_print_plain(void)
{
- const char *rev = ctx.qry.sha1;
+ const char *rev = ctx.qry.oid;
struct object_id oid;
struct commit *commit;
struct pathspec_item path_items = {
@@ -185,7 +185,7 @@ void cgit_print_plain(void)
cgit_print_error_page(404, "Not found", "Not found");
return;
}
- commit = lookup_commit_reference(&oid);
+ commit = lookup_commit_reference(the_repository, &oid);
if (!commit || parse_commit(commit)) {
cgit_print_error_page(404, "Not found", "Not found");
return;
@@ -193,12 +193,13 @@ void cgit_print_plain(void)
if (!path_items.match) {
path_items.match = "";
walk_tree_ctx.match_baselen = -1;
- print_dir(&commit->maybe_tree->object.oid, "", 0, "");
+ print_dir(get_commit_tree_oid(commit), "", 0, "");
walk_tree_ctx.match = 2;
}
else
walk_tree_ctx.match_baselen = basedir_len(path_items.match);
- read_tree_recursive(commit->maybe_tree, "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.match)
cgit_print_error_page(404, "Not found", "Not found");
else if (walk_tree_ctx.match == 2)
diff --git a/ui-refs.c b/ui-refs.c
index 2ec3858..456f610 100644
--- a/ui-refs.c
+++ b/ui-refs.c
@@ -136,7 +136,7 @@ static int print_tag(struct refinfo *ref)
return 0;
}
-static void print_refs_link(char *path)
+static void print_refs_link(const char *path)
{
html("<tr class='nohover'><td colspan='5'>");
cgit_refs_link("[...]", NULL, NULL, ctx.qry.head, NULL, path);
diff --git a/ui-repolist.c b/ui-repolist.c
index 41424c0..529a203 100644
--- a/ui-repolist.c
+++ b/ui-repolist.c
@@ -11,7 +11,7 @@
#include "html.h"
#include "ui-shared.h"
-static time_t read_agefile(char *path)
+static time_t read_agefile(const char *path)
{
time_t result;
size_t size;
@@ -20,7 +20,7 @@ static time_t read_agefile(char *path)
if (readfile(path, &buf, &size)) {
free(buf);
- return -1;
+ return 0;
}
if (parse_date(buf, &date_buf) == 0)
diff --git a/ui-shared.c b/ui-shared.c
index 739505a..acd8ab5 100644
--- a/ui-shared.c
+++ b/ui-shared.c
@@ -22,10 +22,11 @@ static char *http_date(time_t t)
static char month[][4] =
{"Jan", "Feb", "Mar", "Apr", "May", "Jun",
"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
- struct tm *tm = gmtime(&t);
- return fmt("%s, %02d %s %04d %02d:%02d:%02d GMT", day[tm->tm_wday],
- tm->tm_mday, month[tm->tm_mon], 1900 + tm->tm_year,
- tm->tm_hour, tm->tm_min, tm->tm_sec);
+ struct tm tm;
+ gmtime_r(&t, &tm);
+ return fmt("%s, %02d %s %04d %02d:%02d:%02d GMT", day[tm.tm_wday],
+ tm.tm_mday, month[tm.tm_mon], 1900 + tm.tm_year,
+ tm.tm_hour, tm.tm_min, tm.tm_sec);
}
void cgit_print_error(const char *fmt, ...)
@@ -68,15 +69,48 @@ char *cgit_hosturl(void)
char *cgit_currenturl(void)
{
const char *root = cgit_rooturl();
- size_t len = strlen(root);
if (!ctx.qry.url)
return xstrdup(root);
- if (len && root[len - 1] == '/')
+ if (root[0] && root[strlen(root) - 1] == '/')
return fmtalloc("%s%s", root, ctx.qry.url);
return fmtalloc("%s/%s", root, ctx.qry.url);
}
+char *cgit_currentfullurl(void)
+{
+ const char *root = cgit_rooturl();
+ const char *orig_query = ctx.env.query_string ? ctx.env.query_string : "";
+ size_t len = strlen(orig_query);
+ char *query = xmalloc(len + 2), *start_url, *ret;
+
+ /* Remove all url=... parts from query string */
+ memcpy(query + 1, orig_query, len + 1);
+ query[0] = '?';
+ start_url = query;
+ while ((start_url = strstr(start_url, "url=")) != NULL) {
+ if (start_url[-1] == '?' || start_url[-1] == '&') {
+ const char *end_url = strchr(start_url, '&');
+ if (end_url)
+ memmove(start_url, end_url + 1, strlen(end_url));
+ else
+ start_url[0] = '\0';
+ } else
+ ++start_url;
+ }
+ if (!query[1])
+ query[0] = '\0';
+
+ if (!ctx.qry.url)
+ ret = fmtalloc("%s%s", root, query);
+ else if (root[0] && root[strlen(root) - 1] == '/')
+ ret = fmtalloc("%s%s%s", root, ctx.qry.url, query);
+ else
+ ret = fmtalloc("%s/%s%s", root, ctx.qry.url, query);
+ free(query);
+ return ret;
+}
+
const char *cgit_rooturl(void)
{
if (ctx.cfg.virtual_root)
@@ -488,45 +522,45 @@ static void cgit_self_link(char *name, const char *title, const char *class)
else if (!strcmp(ctx.qry.page, "summary"))
cgit_summary_link(name, title, class, ctx.qry.head);
else if (!strcmp(ctx.qry.page, "tag"))
- cgit_tag_link(name, title, class, ctx.qry.has_sha1 ?
- ctx.qry.sha1 : ctx.qry.head);
+ cgit_tag_link(name, title, class, ctx.qry.has_oid ?
+ ctx.qry.oid : ctx.qry.head);
else if (!strcmp(ctx.qry.page, "tree"))
cgit_tree_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "plain"))
cgit_plain_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "blame"))
cgit_blame_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "log"))
cgit_log_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path, ctx.qry.ofs,
ctx.qry.grep, ctx.qry.search,
ctx.qry.showmsg, ctx.qry.follow);
else if (!strcmp(ctx.qry.page, "commit"))
cgit_commit_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "patch"))
cgit_patch_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "refs"))
cgit_refs_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "snapshot"))
cgit_snapshot_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "diff"))
cgit_diff_link(name, title, class, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.sha2,
+ ctx.qry.oid, ctx.qry.oid2,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "stats"))
cgit_stats_link(name, title, class, ctx.qry.head,
@@ -885,10 +919,10 @@ void cgit_add_hidden_formfields(int incl_head, int incl_search,
strcmp(ctx.qry.head, ctx.repo->defbranch))
html_hidden("h", ctx.qry.head);
- if (ctx.qry.sha1)
- html_hidden("id", ctx.qry.sha1);
- if (ctx.qry.sha2)
- html_hidden("id2", ctx.qry.sha2);
+ if (ctx.qry.oid)
+ html_hidden("id", ctx.qry.oid);
+ if (ctx.qry.oid2)
+ html_hidden("id2", ctx.qry.oid2);
if (ctx.qry.showmsg)
html_hidden("showmsg", "1");
@@ -912,12 +946,13 @@ static void cgit_print_path_crumbs(char *path)
{
char *old_path = ctx.qry.path;
char *p = path, *q, *end = path + strlen(path);
+ int levels = 0;
ctx.qry.path = NULL;
cgit_self_link("root", NULL, NULL);
ctx.qry.path = p = path;
while (p < end) {
- if (!(q = strchr(p, '/')))
+ if (!(q = strchr(p, '/')) || levels > 15)
q = end;
*q = '\0';
html_txt("/");
@@ -925,6 +960,7 @@ static void cgit_print_path_crumbs(char *path)
if (q < end)
*q = '/';
p = q + 1;
+ ++levels;
}
ctx.qry.path = old_path;
}
@@ -1003,20 +1039,20 @@ void cgit_print_pageheader(void)
cgit_summary_link("summary", NULL, hc("summary"),
ctx.qry.head);
cgit_refs_link("refs", NULL, hc("refs"), ctx.qry.head,
- ctx.qry.sha1, NULL);
+ ctx.qry.oid, NULL);
cgit_log_link("log", NULL, hc("log"), ctx.qry.head,
NULL, ctx.qry.vpath, 0, NULL, NULL,
ctx.qry.showmsg, ctx.qry.follow);
if (ctx.qry.page && !strcmp(ctx.qry.page, "blame"))
cgit_blame_link("blame", NULL, hc("blame"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.vpath);
else
cgit_tree_link("tree", NULL, hc("tree"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.vpath);
cgit_commit_link("commit", NULL, hc("commit"),
- ctx.qry.head, ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.head, ctx.qry.oid, ctx.qry.vpath);
cgit_diff_link("diff", NULL, hc("diff"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.sha2, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.oid2, ctx.qry.vpath);
if (ctx.repo->max_stats)
cgit_stats_link("stats", NULL, hc("stats"),
ctx.qry.head, ctx.qry.vpath);
@@ -1159,31 +1195,17 @@ void cgit_print_snapshot_links(const struct cgit_repo *repo, const char *ref,
void cgit_set_title_from_path(const char *path)
{
- size_t path_len, path_index, path_last_end;
- char *new_title;
+ struct strbuf sb = STRBUF_INIT;
+ const char *slash, *last_slash;
if (!path)
return;
- path_len = strlen(path);
- new_title = xmalloc(path_len + 3 + strlen(ctx.page.title) + 1);
- new_title[0] = '\0';
-
- for (path_index = path_len, path_last_end = path_len; path_index-- > 0;) {
- if (path[path_index] == '/') {
- if (path_index == path_len - 1) {
- path_last_end = path_index - 1;
- continue;
- }
- strncat(new_title, &path[path_index + 1], path_last_end - path_index - 1);
- strcat(new_title, "\\");
- path_last_end = path_index;
- }
+ for (last_slash = path + strlen(path); (slash = memrchr(path, '/', last_slash - path)) != NULL; last_slash = slash) {
+ strbuf_add(&sb, slash + 1, last_slash - slash - 1);
+ strbuf_addstr(&sb, " \xc2\xab ");
}
- if (path_last_end)
- strncat(new_title, path, path_last_end);
-
- strcat(new_title, " - ");
- strcat(new_title, ctx.page.title);
- ctx.page.title = new_title;
+ strbuf_add(&sb, path, last_slash - path);
+ strbuf_addf(&sb, " - %s", ctx.page.title);
+ ctx.page.title = strbuf_detach(&sb, NULL);
}
diff --git a/ui-shared.h b/ui-shared.h
index 4d5978b..6964873 100644
--- a/ui-shared.h
+++ b/ui-shared.h
@@ -5,6 +5,7 @@ extern const char *cgit_httpscheme(void);
extern char *cgit_hosturl(void);
extern const char *cgit_rooturl(void);
extern char *cgit_currenturl(void);
+extern char *cgit_currentfullurl(void);
extern const char *cgit_loginurl(void);
extern char *cgit_repourl(const char *reponame);
extern char *cgit_fileurl(const char *reponame, const char *pagename,
diff --git a/ui-ssdiff.c b/ui-ssdiff.c
index 68c2044..af8bc9e 100644
--- a/ui-ssdiff.c
+++ b/ui-ssdiff.c
@@ -103,8 +103,7 @@ static int line_from_hunk(char *line, char type)
return 0;
len = buf2 - buf1;
buf2 = xmalloc(len + 1);
- strncpy(buf2, buf1, len);
- buf2[len] = '\0';
+ strlcpy(buf2, buf1, len + 1);
res = atoi(buf2);
free(buf2);
return res;
@@ -118,6 +117,7 @@ static char *replace_tabs(char *line)
int n_tabs = 0;
int i;
char *result;
+ size_t result_len;
if (linelen == 0) {
result = xmalloc(1);
@@ -129,16 +129,19 @@ static char *replace_tabs(char *line)
if (line[i] == '\t')
n_tabs += 1;
}
- result = xmalloc(linelen + n_tabs * 8 + 1);
+ result_len = linelen + n_tabs * 8;
+ result = xmalloc(result_len + 1);
result[0] = '\0';
for (;;) {
cur_buf = strchr(prev_buf, '\t');
if (!cur_buf) {
- strcat(result, prev_buf);
+ linelen = strlen(result);
+ strlcpy(&result[linelen], prev_buf, result_len - linelen + 1);
break;
} else {
- strncat(result, prev_buf, cur_buf - prev_buf);
+ linelen = strlen(result);
+ strlcpy(&result[linelen], prev_buf, cur_buf - prev_buf + 1);
linelen = strlen(result);
memset(&result[linelen], ' ', 8 - (linelen % 8));
result[linelen + 8 - (linelen % 8)] = '\0';
@@ -206,11 +209,13 @@ static void print_part_with_lcs(char *class, char *line, char *lcs)
}
} else if (line[i] == lcs[j]) {
same = 1;
- htmlf("</span>");
+ html("</span>");
j += 1;
}
html_txt(c);
}
+ if (!same)
+ html("</span>");
}
static void print_ssdiff_line(char *class,
@@ -235,7 +240,7 @@ static void print_ssdiff_line(char *class,
char *fileurl = cgit_fileurl(ctx.repo->url, "tree", old_file->path, id_str);
html("<td class='lineno'><a href='");
html(fileurl);
- htmlf("' id='%s'>%s</a>", lineno_str, lineno_str + 1);
+ htmlf("'>%s</a>", lineno_str + 1);
html("</td>");
htmlf("<td class='%s'>", class);
free(fileurl);
@@ -258,7 +263,7 @@ static void print_ssdiff_line(char *class,
char *fileurl = cgit_fileurl(ctx.repo->url, "tree", new_file->path, id_str);
html("<td class='lineno'><a href='");
html(fileurl);
- htmlf("' id='%s'>%s</a>", lineno_str, lineno_str + 1);
+ htmlf("'>%s</a>", lineno_str + 1);
html("</td>");
htmlf("<td class='%s'>", class);
free(fileurl);
@@ -404,7 +409,7 @@ void cgit_ssdiff_header_begin(void)
void cgit_ssdiff_header_end(void)
{
- html("</td><tr>");
+ html("</td></tr>");
}
void cgit_ssdiff_footer(void)
diff --git a/ui-stats.c b/ui-stats.c
index 7acd358..09b3625 100644
--- a/ui-stats.c
+++ b/ui-stats.c
@@ -166,7 +166,7 @@ static void add_commit(struct string_list *authors, struct commit *commit,
struct authorstat *authorstat;
struct string_list *items;
char *tmp;
- struct tm *date;
+ struct tm date;
time_t t;
uintptr_t *counter;
@@ -180,9 +180,9 @@ static void add_commit(struct string_list *authors, struct commit *commit,
authorstat = author->util;
items = &authorstat->list;
t = info->committer_date;
- date = gmtime(&t);
- period->trunc(date);
- tmp = xstrdup(period->pretty(date));
+ gmtime_r(&t, &date);
+ period->trunc(&date);
+ tmp = xstrdup(period->pretty(&date));
item = string_list_insert(items, tmp);
counter = (uintptr_t *)&item->util;
if (*counter)
@@ -215,15 +215,15 @@ static struct string_list collect_stats(const struct cgit_period *period)
int argc = 3;
time_t now;
long i;
- struct tm *tm;
+ struct tm tm;
char tmp[11];
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
- strftime(tmp, sizeof(tmp), "%Y-%m-%d", tm);
+ period->dec(&tm);
+ strftime(tmp, sizeof(tmp), "%Y-%m-%d", &tm);
argv[2] = xstrdup(fmt("--since=%s", tmp));
if (ctx.qry.path) {
argv[3] = "--";
@@ -241,7 +241,7 @@ static struct string_list collect_stats(const struct cgit_period *period)
memset(&authors, 0, sizeof(authors));
while ((commit = get_revision(&rev)) != NULL) {
add_commit(&authors, commit, period);
- free_commit_buffer(commit);
+ free_commit_buffer(the_repository->parsed_objects, commit);
free_commit_list(commit->parents);
commit->parents = NULL;
}
@@ -261,21 +261,21 @@ static void print_combined_authorrow(struct string_list *authors, int from,
struct string_list_item *date;
time_t now;
long i, j, total, subtotal;
- struct tm *tm;
+ struct tm tm;
char *tmp;
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
+ period->dec(&tm);
total = 0;
htmlf("<tr><td class='%s'>%s</td>", leftclass,
fmt(name, to - from + 1));
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
- period->inc(tm);
+ tmp = period->pretty(&tm);
+ period->inc(&tm);
subtotal = 0;
for (i = from; i <= to; i++) {
author = &authors->items[i];
@@ -300,20 +300,20 @@ static void print_authors(struct string_list *authors, int top,
struct string_list_item *date;
time_t now;
long i, j, total;
- struct tm *tm;
+ struct tm tm;
char *tmp;
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
+ period->dec(&tm);
html("<table class='stats'><tr><th>Author</th>");
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
+ tmp = period->pretty(&tm);
htmlf("<th>%s</th>", tmp);
- period->inc(tm);
+ period->inc(&tm);
}
html("<th>Total</th></tr>\n");
@@ -329,10 +329,10 @@ static void print_authors(struct string_list *authors, int top,
items = &authorstat->list;
total = 0;
for (j = 0; j < period->count; j++)
- period->dec(tm);
+ period->dec(&tm);
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
- period->inc(tm);
+ tmp = period->pretty(&tm);
+ period->inc(&tm);
date = string_list_lookup(items, tmp);
if (!date)
html("<td>0</td>");
diff --git a/ui-summary.c b/ui-summary.c
index 8e81ac4..947812a 100644
--- a/ui-summary.c
+++ b/ui-summary.c
@@ -99,7 +99,7 @@ static char* append_readme_path(const char *filename, const char *ref, const cha
return full_path;
}
-void cgit_print_repo_readme(char *path)
+void cgit_print_repo_readme(const char *path)
{
char *filename, *ref, *mimetype;
int free_filename = 0;
diff --git a/ui-summary.h b/ui-summary.h
index 0896650..cba696a 100644
--- a/ui-summary.h
+++ b/ui-summary.h
@@ -2,6 +2,6 @@
#define UI_SUMMARY_H
extern void cgit_print_summary(void);
-extern void cgit_print_repo_readme(char *path);
+extern void cgit_print_repo_readme(const char *path);
#endif /* UI_SUMMARY_H */
diff --git a/ui-tag.c b/ui-tag.c
index 2c96c37..424bbcc 100644
--- a/ui-tag.c
+++ b/ui-tag.c
@@ -33,7 +33,7 @@ static void print_tag_content(char *buf)
static void print_download_links(char *revname)
{
- html("<tr><th>download</th><td class='sha1'>");
+ html("<tr><th>download</th><td class='oid'>");
cgit_print_snapshot_links(ctx.repo, revname, "<br/>");
html("</td></tr>");
}
@@ -53,7 +53,7 @@ void cgit_print_tag(char *revname)
"Bad tag reference: %s", revname);
goto cleanup;
}
- obj = parse_object(&oid);
+ obj = parse_object(the_repository, &oid);
if (!obj) {
cgit_print_error_page(500, "Internal server error",
"Bad object id: %s", oid_to_hex(&oid));
@@ -63,7 +63,7 @@ void cgit_print_tag(char *revname)
struct tag *tag;
struct taginfo *info;
- tag = lookup_tag(&oid);
+ tag = lookup_tag(the_repository, &oid);
if (!tag || parse_tag(tag) || !(info = cgit_parse_tag(tag))) {
cgit_print_error_page(500, "Internal server error",
"Bad tag object: %s", revname);
@@ -71,7 +71,7 @@ void cgit_print_tag(char *revname)
}
cgit_print_layout_start();
html("<table class='commit-info'>\n");
- htmlf("<tr><td>tag name</td><td>");
+ html("<tr><td>tag name</td><td>");
html_txt(revname);
htmlf(" (%s)</td></tr>\n", oid_to_hex(&oid));
if (info->tagger_date > 0) {
@@ -91,7 +91,7 @@ void cgit_print_tag(char *revname)
cgit_close_filter(ctx.repo->email_filter);
html("</td></tr>\n");
}
- html("<tr><td>tagged object</td><td class='sha1'>");
+ html("<tr><td>tagged object</td><td class='oid'>");
cgit_object_link(tag->tagged);
html("</td></tr>\n");
if (ctx.repo->snapshots)
@@ -103,10 +103,10 @@ void cgit_print_tag(char *revname)
} else {
cgit_print_layout_start();
html("<table class='commit-info'>\n");
- htmlf("<tr><td>tag name</td><td>");
+ html("<tr><td>tag name</td><td>");
html_txt(revname);
html("</td></tr>\n");
- html("<tr><td>tagged object</td><td class='sha1'>");
+ html("<tr><td>tagged object</td><td class='oid'>");
cgit_object_link(obj);
html("</td></tr>\n");
if (ctx.repo->snapshots)
diff --git a/ui-tree.c b/ui-tree.c
index e6b3074..b61f6f5 100644
--- a/ui-tree.c
+++ b/ui-tree.c
@@ -84,7 +84,7 @@ static void print_binary_buffer(char *buf, unsigned long size)
html("</table>\n");
}
-static void print_object(const struct object_id *oid, char *path, const char *basename, const char *rev)
+static void print_object(const struct object_id *oid, const char *path, const char *basename, const char *rev)
{
enum object_type type;
char *buf;
@@ -110,7 +110,7 @@ static void print_object(const struct object_id *oid, char *path, const char *ba
htmlf("blob: %s (", oid_to_hex(oid));
cgit_plain_link("plain", NULL, NULL, ctx.qry.head,
rev, path);
- if (ctx.cfg.enable_blame) {
+ if (ctx.repo->enable_blame) {
html(") (");
cgit_blame_link("blame", NULL, NULL, ctx.qry.head,
rev, path);
@@ -139,8 +139,7 @@ struct single_tree_ctx {
};
static int single_tree_cb(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage,
- void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct single_tree_ctx *ctx = cbdata;
@@ -177,7 +176,7 @@ static void write_tree_link(const struct object_id *oid, char *name,
cgit_tree_link(name, NULL, "ls-dir", ctx.qry.head, rev,
fullpath->buf);
- tree = lookup_tree(&tree_ctx.oid);
+ tree = lookup_tree(the_repository, &tree_ctx.oid);
if (!tree)
return;
@@ -185,8 +184,7 @@ static void write_tree_link(const struct object_id *oid, char *name,
tree_ctx.name = NULL;
tree_ctx.count = 0;
- read_tree_recursive(tree, "", 0, 1, &paths, single_tree_cb,
- &tree_ctx);
+ read_tree(the_repository, tree, &paths, single_tree_cb, &tree_ctx);
if (tree_ctx.count != 1)
break;
@@ -199,7 +197,7 @@ static void write_tree_link(const struct object_id *oid, char *name,
}
static int ls_item(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
char *name;
@@ -251,7 +249,7 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
if (!S_ISGITLINK(mode))
cgit_plain_link("plain", NULL, "button", ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
- if (!S_ISDIR(mode) && ctx.cfg.enable_blame)
+ if (!S_ISDIR(mode) && ctx.repo->enable_blame)
cgit_blame_link("blame", NULL, "button", ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
html("</td></tr>\n");
@@ -279,7 +277,7 @@ static void ls_tail(void)
cgit_print_layout_end();
}
-static void ls_tree(const struct object_id *oid, char *path, struct walk_tree_context *walk_tree_ctx)
+static void ls_tree(const struct object_id *oid, const char *path, struct walk_tree_context *walk_tree_ctx)
{
struct tree *tree;
struct pathspec paths = {
@@ -294,13 +292,13 @@ static void ls_tree(const struct object_id *oid, char *path, struct walk_tree_co
}
ls_head();
- read_tree_recursive(tree, "", 0, 1, &paths, ls_item, walk_tree_ctx);
+ read_tree(the_repository, tree, &paths, ls_item, walk_tree_ctx);
ls_tail();
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -325,7 +323,7 @@ static int walk_tree(const struct object_id *oid, struct strbuf *base,
return 0;
}
}
- ls_item(oid, base, pathname, mode, stage, walk_tree_ctx);
+ ls_item(oid, base, pathname, mode, walk_tree_ctx);
return 0;
}
@@ -359,7 +357,7 @@ void cgit_print_tree(const char *rev, char *path)
"Invalid revision name: %s", rev);
return;
}
- commit = lookup_commit_reference(&oid);
+ commit = lookup_commit_reference(the_repository, &oid);
if (!commit || parse_commit(commit)) {
cgit_print_error_page(404, "Not found",
"Invalid commit reference: %s", rev);
@@ -369,11 +367,12 @@ void cgit_print_tree(const char *rev, char *path)
walk_tree_ctx.curr_rev = xstrdup(rev);
if (path == NULL) {
- ls_tree(&commit->maybe_tree->object.oid, NULL, &walk_tree_ctx);
+ ls_tree(get_commit_tree_oid(commit), NULL, &walk_tree_ctx);
goto cleanup;
}
- read_tree_recursive(commit->maybe_tree, "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (walk_tree_ctx.state == 1)
ls_tail();
else if (walk_tree_ctx.state == 2)
diff --git a/ui-snapshot.c b/ui-snapshot.c
index fa3ceaf..18361a6 100644
--- a/ui-snapshot.c
+++ b/ui-snapshot.c
@@ -13,32 +13,32 @@
static int write_archive_type(const char *format, const char *hex, const char *prefix)
{
- struct argv_array argv = ARGV_ARRAY_INIT;
+ struct strvec argv = STRVEC_INIT;
const char **nargv;
int result;
- argv_array_push(&argv, "snapshot");
- argv_array_push(&argv, format);
+ strvec_push(&argv, "snapshot");
+ strvec_push(&argv, format);
if (prefix) {
struct strbuf buf = STRBUF_INIT;
strbuf_addstr(&buf, prefix);
strbuf_addch(&buf, '/');
- argv_array_push(&argv, "--prefix");
- argv_array_push(&argv, buf.buf);
+ strvec_push(&argv, "--prefix");
+ strvec_push(&argv, buf.buf);
strbuf_release(&buf);
}
- argv_array_push(&argv, hex);
+ strvec_push(&argv, hex);
/*
* Now we need to copy the pointers to arguments into a new
* structure because write_archive will rearrange its arguments
* which may result in duplicated/missing entries causing leaks
- * or double-frees in argv_array_clear.
+ * or double-frees in strvec_clear.
*/
- nargv = xmalloc(sizeof(char *) * (argv.argc + 1));
- /* argv_array guarantees a trailing NULL entry. */
- memcpy(nargv, argv.argv, sizeof(char *) * (argv.argc + 1));
+ nargv = xmalloc(sizeof(char *) * (argv.nr + 1));
+ /* strvec guarantees a trailing NULL entry. */
+ memcpy(nargv, argv.v, sizeof(char *) * (argv.nr + 1));
- result = write_archive(argv.argc, nargv, NULL, NULL, 0);
- argv_array_clear(&argv);
+ result = write_archive(argv.nr, nargv, NULL, the_repository, NULL, 0);
+ strvec_clear(&argv);
free(nargv);
return result;
}
@@ -79,18 +79,32 @@ static int write_tar_bzip2_archive(const char *hex, const char *prefix)
return write_compressed_tar_archive(hex, prefix, argv);
}
+static int write_tar_lzip_archive(const char *hex, const char *prefix)
+{
+ char *argv[] = { "lzip", NULL };
+ return write_compressed_tar_archive(hex, prefix, argv);
+}
+
static int write_tar_xz_archive(const char *hex, const char *prefix)
{
char *argv[] = { "xz", NULL };
return write_compressed_tar_archive(hex, prefix, argv);
}
+static int write_tar_zstd_archive(const char *hex, const char *prefix)
+{
+ char *argv[] = { "zstd", "-T0", NULL };
+ return write_compressed_tar_archive(hex, prefix, argv);
+}
+
const struct cgit_snapshot_format cgit_snapshot_formats[] = {
/* .tar must remain the 0 index */
{ ".tar", "application/x-tar", write_tar_archive },
{ ".tar.gz", "application/x-gzip", write_tar_gzip_archive },
{ ".tar.bz2", "application/x-bzip2", write_tar_bzip2_archive },
+ { ".tar.lz", "application/x-lzip", write_tar_lzip_archive },
{ ".tar.xz", "application/x-xz", write_tar_xz_archive },
+ { ".tar.zst", "application/x-zstd", write_tar_zstd_archive },
{ ".zip", "application/x-zip", write_zip_archive },
{ NULL }
};
@@ -147,7 +161,7 @@ static int make_snapshot(const struct cgit_snapshot_format *format,
"Bad object id: %s", hex);
return 1;
}
- if (!lookup_commit_reference(&oid)) {
+ if (!lookup_commit_reference(the_repository, &oid)) {
cgit_print_error_page(400, "Bad request",
"Not a commit reference: %s", hex);
return 1;
@@ -156,6 +170,7 @@ static int make_snapshot(const struct cgit_snapshot_format *format,
ctx.page.mimetype = xstrdup(format->mimetype);
ctx.page.filename = xstrdup(filename);
cgit_print_http_headers();
+ init_archivers();
format->write_func(hex, prefix);
return 0;
}